# -*- coding: utf-8 -*-
# Package information, generated from cs_package.py.in by make.

#-------------------------------------------------------------------------------

# This file is part of Code_Saturne, a general-purpose CFD tool.
#
# Copyright (C) 1998-2021 EDF S.A.
#
# This program is free software; you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation; either version 2 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
# details.
#
# You should have received a copy of the GNU General Public License along with
# this program; if not, write to the Free Software Foundation, Inc., 51 Franklin
# Street, Fifth Floor, Boston, MA 02110-1301, USA.

#-------------------------------------------------------------------------------

import os
import sys

try:
    from code_saturne import cs_config
except Exception:
    import cs_config

# Package information
#--------------------

class package:

    def __init__(self,
                 scriptdir=None,
                 reload_config=False,
                 name="code_saturne"):

        base_package = 'code_saturne'

        # System configuration (compilers, pre-requisites, ...)

        if reload_config:
            import importlib
            try:
                config = importlib.import_module('cs_config', package=base_package)
                self.config = config.config()
            except Exception:  # Upon installation
                import traceback
                exc_type, exc_value, exc_traceback = sys.exc_info()
                traceback.print_exception(exc_type, exc_value, exc_traceback,
                                          limit=2, file=sys.stderr)
        else:
            self.config = cs_config.config()

        # Package information

        self.name = name
        self.tarname = "code_saturne"
        self.pkgversion = "7.0.2"
        self.string = "code_saturne 7.0.2"
        self.bugreport = "saturne-support@edf.fr"
        self.url = "https://code-saturne.org"
        self.version = "7.0.2"
        self.version_full = "7.0.2-d029e16a2"
        self.version_short = "7.0"
        self.revision = "d029e16a2"

        self.code_name = "Code_Saturne"

        self.preprocessor = "cs_preprocess" + self.config.exeext
        self.solver = self.config.solver_modules[name]['solver']
        self.check_syntax = "cs_check_syntax" + self.config.exeext
        self.io_dump = "cs_io_dump" + self.config.exeext
        self.runcase = "runcase" + self.config.shext
        self.runsolver = "run_solver" + self.config.shext
        self.configfile = "code_saturne" + self.config.cfgext
        self.scratchdir = 'tmp_Saturne'

        # Installation directories

        self.dirs = {'prefix': ("", "/usr"),
                     'exec_prefix': ("", "/usr"),
                     'bindir': ("bin", "/usr/bin"),
                     'includedir': ("include", "/usr/include"),
                     'pkgincludedir': (os.path.join("include", base_package),
                                       "/usr/include/code_saturne"),
                     'libdir': ("lib", "/usr/lib"),
                     'libexecdir': ("libexec", "/usr/libexec"),
                     'pkglibexecdir': (os.path.join("libexec", base_package),
                                       "/usr/libexec/code_saturne"),
                     'pythondir': (os.path.join("lib",
                                                "python" + sys.version[:3],
                                                "site-packages"),
                                   "/usr/lib/python3.11/site-packages"),
                     'pkgpythondir': (os.path.join("lib",
                                                   "python" + sys.version[:3],
                                                   "site-packages",
                                                   base_package),
                                      "/usr/lib/python3.11/site-packages/code_saturne"),
                     'localedir': (os.path.join("share", "locale"),
                                   "/usr/share/locale"),
                     'datarootdir': ("share", "/usr/share"),
                     'datadir': ("share", "/usr/share"),
                     'pkgdatadir': (os.path.join("share", base_package),
                                    "/usr/share/code_saturne"),
                     'docdir': (os.path.join("share", "doc", self.name),
                                "/usr/share/doc/code_saturne"),
                     'pdfdir': (os.path.join("share", "doc", self.name),
                                "/usr/share/doc/code_saturne"),
                     'sysconfdir': ("etc", "/usr/etc")}

        # Adjust docdir for additional modules.

        # We should try to find a cleaner/more consistant solution,
        # perhaps storing all documents in the main module's
        # directory (code_saturne).

        docdir_1 = self.dirs['docdir'][1]
        docdir_1 = os.path.join(os.path.split(docdir_1)[0], name)
        self.dirs['docdir'] = (self.dirs['docdir'][0],
                               docdir_1)

    def get_dir(self, installdir):

        # First, handle the standard "non relocatable" case
        if self.config.features['relocatable'] == "no":
            return self.dirs[installdir][1]

        # Second, check if CS_ROOT_DIR was forced, or if CS_ROOT_DIR is set
        if 'cs_root_dir' in self.dirs:
            prefix = self.dirs['cs_root_dir']
        else:
            prefix = os.getenv('CS_ROOT_DIR')

        # Last, assume that the standard tree structure is used
        if prefix == None:
            prefix = os.path.dirname(os.path.realpath(__file__))
            i = prefix.find(self.dirs['pythondir'][0])
            if i > -1:
                prefix = os.path.dirname(prefix[:i+1])

        return os.path.join(prefix, self.dirs[installdir][0])

    def get_preprocessor(self):

        if self.config.features['frontend'] == "no":
            raise Exception("This " + self.name + " build does not " + \
                            "include the front-end and Preprocessor.")
        return os.path.join(self.get_dir("pkglibexecdir"),
                            self.preprocessor)

    def get_io_dump(self):

        return os.path.join(self.get_dir("pkglibexecdir"),
                            self.io_dump)

    def get_solver(self):

        return os.path.join(self.get_dir("pkglibexecdir"),
                            self.solver)

    def get_check_syntax(self):

        return os.path.join(self.get_dir("pkglibexecdir"),
                            self.check_syntax)

    def get_io_dump(self):

        return os.path.join(self.get_dir("pkglibexecdir"),
                            self.io_dump)

    def get_global_configfile(self):

        # Windows:         C:\ProgramData
        # Linux and co:    /etc

        if sys.platform.startswith("win"):
            configdir = os.path.join(os.getenv("PROGRAMDATA"),
                                     self.code_name, self.version_short)
        else:
            configdir = self.get_dir("sysconfdir")

        return [os.path.join(configdir, self.configfile)]

    def get_user_configfile(self):

        # Windows:         C:\Users\{user}\AppData\Roaming
        # Linux and co:    /home/{user}   (or similar)

        if sys.platform.startswith("win"):
            configdir = os.path.join(os.getenv("APPDATA"),
                                     self.code_name, self.version_short)
            return [os.path.join(configdir, self.configfile)]
        else:
            configdir = os.path.expanduser("~")
            return [os.path.join(configdir, "." + self.configfile)]

    def get_configfiles(self):

        u_cfg = self.get_user_configfile()
        g_cfg = self.get_global_configfile()

        return g_cfg + u_cfg

    def get_batchdir(self):

        return os.path.join(self.get_dir("pkgdatadir"),
                            'batch')

    def get_pkgdatadir_script(self, script):

        return os.path.join(self.get_dir("pkgdatadir"),
                            script)

    def get_alternate_version(self, version):
        """
        Return alternate version package object
        """

        if not version:
            return self

        pkg = None

        # Determine path (by absolute or local name)
        pythondir = os.path.normpath(version)
        prefix = os.path.normpath(self.get_dir("exec_prefix"))
        if self.get_dir("pkgpythondir").find(prefix) > -1:
            postfix = self.get_dir("pkgpythondir")[len(prefix)+1:]
            if not os.path.isabs(pythondir):
                prefix = os.path.split(prefix)[0]
            else:
                prefix = version
            pythondir = os.path.normpath(os.path.join(prefix,
                                                      version,
                                                      postfix))

        # load alternate package
        if os.path.isdir(pythondir):
            import sys
            sys.path.insert(0, pythondir)
            if sys.version < '3.4':
                from code_saturne import cs_config
                reload(cs_config)
                import cs_package
                reload(cs_package)
                pkg = cs_package.package(name=self.name)
            else:
                import importlib
                importlib.invalidate_caches()
                try:
                    package = importlib.import_module('cs_package',
                                                      package='code_saturne')
                    pkg = package.package(reload_config=True,
                                          name=self.name)
                except Exception:
                    import traceback
                    exc_type, exc_value, exc_traceback = sys.exc_info()
                    traceback.print_exception(exc_type, exc_value, exc_traceback,
                                              limit=2, file=sys.stderr)
                    pkg = self
            sys.path.pop(0)
            if pkg.config.features['relocatable'] != "no":
                pkg.dirs['cs_root_dir'] = os.path.normpath(os.path.join(pythondir,
                                                           '..', '..', '..', '..'))
        else:
            raise ImportError("Alternative version '" + pythondir + "' not found.")

        return pkg

    def get_cross_compile(self):
        """
        Return cross-conpilation info
        """
        return self.config.features['build_os']

#-------------------------------------------------------------------------------
